vec3 colorPM = pow2(colorP.rgb);
vec3 glColorM = glColor.rgb;
glColorM.g = max(glColorM.g, 0.35);
color.rgb = colorPM * sqrt1(glColorM) * vec3(1.0, 0.85, 0.8);

#ifdef GBUFFERS_WATER
    translucentMultAlreadyCalculated = true;

	#ifdef GENERATED_NORMALS
		noGeneratedNormals = true;
	#endif

    lmCoordM.y = min(lmCoord.y * 1.07, 1.0); // Iris/Sodium skylight inconsistency workaround

    #if WATER_QUALITY >= 2 || defined WATER_WAVES
        vec3 worldPos = playerPos + cameraPosition;
        vec2 waterPos = (floor(worldPos.xz * 16.0) + worldPos.y * 32.0) * 0.002;
        vec2 wind = vec2(frameTimeCounter * 0.016, 0.0);
    #endif

    #if WATER_QUALITY >= 2
        // Noise Coloring
        float noise = texture2D(noisetex, (waterPos + wind) * 0.25).g;
              noise = noise - 0.5;
              noise *= 0.25;
        color.rgb = pow(color.rgb, vec3(1.0 + noise));
        
        if (isEyeInWater != 1) {
            float depthT = texelFetch(depthtex1, texelCoord, 0).r;
            vec3 screenPosT = vec3(screenCoord, depthT);
            #ifdef TAA
                vec3 viewPosT = ScreenToView(vec3(TAAJitter(screenPosT.xy, -0.5), screenPosT.z));
            #else
                vec3 viewPosT = ScreenToView(screenPosT);
            #endif
            float lViewPosT = length(viewPosT);
            float lViewPosDif = lViewPos - lViewPosT;
            
            // Water Fog
            color.a = sqrt1(color.a);
            color.a *= 0.5 + 0.5 * max0(1.0 - exp(lViewPosDif * 0.1));
        
            // Water Foam
            if (NdotU > 0.99) {
                vec3 matrixM = vec3(
                    gbufferModelViewInverse[0].y,
                    gbufferModelViewInverse[1].y,
                    gbufferModelViewInverse[2].y
                );
                float playerPosTY = dot(matrixM, viewPosT) + gbufferModelViewInverse[3].y;
                float yPosDif = playerPosTY - playerPos.y;

                float dotColorPM = dot(colorPM, colorPM);
                float foamThreshold = min(pow2(dotColorPM) * 1.6, 1.2);
                float foam = pow2(clamp((foamThreshold + yPosDif) / foamThreshold, 0.0, 1.0));
                foam *= color.b * 0.6 + 0.4 * pow2(lmCoord.y);
                foam *= clamp((fract(worldPos.y) - 0.7) * 10.0, 0.0, 1.0);

                color = mix(color, vec4(0.9, 0.95, 1.05, 1.0), foam);
                fresnel = mix(fresnel, 0.0, foam);
            }
        } else {
            color.rgb *= 1.0 + lmCoordM.y * 0.5;
            noDirectionalShading = true;

            fresnel *= 0.5;
        }

        //if (isEyeInWater == 0) color.a = mix(color.a, 1.0, 0.8 * GetWaterFog(lViewPosDif));
    #endif

    #ifdef WATER_WAVES
        vec3 normalMap = vec3(0.0, 0.0, 1.0);
        normalMap.xy += texture2D(noisetex, waterPos + wind * 0.5).rg - 0.5;
             waterPos *= 0.5;
        normalMap.xy -= texture2D(noisetex, waterPos - wind).rg - 0.5;
             waterPos *= 0.5;
        normalMap.xy -= texture2D(noisetex, waterPos + wind).rg - 0.5;

        normalMap.xy *= 0.03 * lmCoordM.y + 0.01;

        mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
                              tangent.y, binormal.y, normal.y,
                              tangent.z, binormal.z, normal.z);

        vec3 minNormal = mix(normal, vec3(-1.0), pow2(1.0 - fresnel));
        normalM = clamp(normalize(normalMap * tbnMatrix), minNormal, vec3(1.0));

        /*vec2 coord = signMidCoordPos * 0.5 + 0.5;
        vec2 absMidCoordPos2 = absMidCoordPos * 2.0;
        vec2 midCoord = texCoord - absMidCoordPos * signMidCoordPos;
        vec2 minimumMidCoordPos = midCoord - absMidCoordPos;

        coord += normalMap.xy * (1.0 + 6.0 * fresnel);
            
        vec2 sampleCoord = fract(coord) * absMidCoordPos2 + minimumMidCoordPos;

        colorP = texture2D(texture, sampleCoord);
        colorPM = pow2(colorP.rgb);
        color.rgb = colorPM * pow(glColor.rgb, vec3(0.35));*/
    #endif

    // Blending 
    float fresnel2 = pow2(fresnel);
    translucentMult.rgb = normalize(sqrt2(glColor.rgb)) * vec3(0.7, 0.7, 1.0);
    translucentMult.rgb = mix(translucentMult.rgb, vec3(float(isEyeInWater == 0) * eyeBrightnessM), fresnel2);
    if (isEyeInWater == 0)
        translucentMult.rgb = mix(translucentMult.rgb, vec3(1.0), min1(lViewPos * TRANSLUCENT_BLEND_FALLOFF_MULT));
    color.a = mix(color.a, 1.0, fresnel2);

    // PBR
    smoothnessG = 0.5;
    highlightMult = min(pow2(pow2(dot(colorP.rgb, colorP.rgb) * 0.4)), 0.5) * (16.0 - 15.0 * fresnel) * (sunVisibility > 0.5 ? 1.0 : 0.5);
#endif
